function [realZa1,imagZa1]=CalcHornAcousticImpedance(fLow,fHigh,delf,ang)
% This function calculates the complex normalized input acoustic 
% impedance Za1 of a multi-segment finite horn assembly consisting of
% cylindrical, conical and/or exponential segments. It outputs the real 
% resistive part, realZa1, and the imaginary reactive part,imagZa1, to 
% the Matlab workspace. It then plots the real resistive plot in blue 
% and the imaginary reactive part in red.
% The user must input four inputs:
%  fLow - the low frequency limit of interest (Hz)
%  fHigh - the high frequecy limit of interest (Hz)
%  delf - the frequency increment desired (Hz)
%  ang - the solid angle of radiation output desired in units of pi
%   ang = 0.5 ; 1/8 space (corner placement)
%   ang = 1.0 ; 1/4 space (wall placement)
%   ang = 2.0 ; 1/2 space (floor placement)
%   ang = 4.0 ; full space (open to full space)
% N.B., ang should only be entered as one of the 
%       four values 0.5, 1.0, 2.0, 4.0 (any other value defaults to 2.0)
%

% 
% ang(units of pi)will contrast the following four termination impedances
% using a modified 'piston in infinite baffle' model:
% (1) Zp = @(k) (rho0*c/S99)*Zpn(2*k*sqrt(4)*sqrt(S99/pi));   % ang=0.5
% (2) Zp = @(k) (rho0*c/S99)*Zpn(2*k*sqrt(2)*sqrt(S99/pi));   % ang=1.0
% (3) Zp = @(k) (rho0*c/S99)*Zpn(2*k*sqrt(S99/pi));           % ang=2.0
% (4) Zp = @(k) (rho0*c/S99)*Zpn(2*k*sqrt(0.5)*sqrt(S99/pi)); % ang=4.0
% where S99 is the mouth area, and Zpn is defined below
% all units are in cgs system
%

%%%%%%%%%%%%%%%%%%%%
% Version 1.0      %
% January 24, 2013 %
% Joe Ladish       %
% joeladis@unm.edu %
%%%%%%%%%%%%%%%%%%%%
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Calculational Example for Reference %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%
% This example calculates a three segment finite horn
% assembly by calling FiniteCylHorn, FiniteConHorn,
% and FiniteExpHorn to obtain their respective
% Transfer matrices. It then multiplies these matrices
% to obtain the final impedance transfer matrix, Ttot.
% 
%
% To execute the included example, at the Matlab command line type
%
%    [realZa1,imagZa1]=CalcHornAcousticImpedance(10,20000,1,0.5);
%
% This corresponds to choosing a low frequency fLow = 10; fHigh = 20000,
% delf = 1, and ang = 0.5
% N.B. the semicolon at the end of the command is suggested, otherwise,
% you will print out all the values of the return variables realZa1
% and imagZa1 (roughly 40000 values!)
%
%
% This example uses a three segment finite horn:
% S1 = segment input area, S2 = segment output area, L = segment length
% Segment 1: Type Cylindrical: S1 = S2 = 5.07, L = 7.62;
% Segment 2: Type Conical: S1 = 5.07, S2 = 45.6, L = 25;
% Segment 3: Type Exponential: S1 = 45.6, S2 = 1140, L = 25;
% N.B., in general segment input area (S1) = segment output area (S2)
%       of preceeding segment;
% first input segment area 'S1' is throat area S1 in impedance calculation
% final output segment area 'S2' is mouth area S99 in impedance calculation
% N.B., this means S99 = 1140 in this example;
%
% Segment 1: throat segment of horn assembly in this example
%
T1=FiniteCylHorn(5.07,7.62);    % call to function FiniteCylHorn
%
% Segment 2: intermediate segment in this example
%
T2=FiniteConHorn(5.07,45.6,25); % call to function FiniteConHorn
%
% Segment 3: mouth segment of horn assembly in this example which
%            terminates in "piston in baffle" impedance
T3=FiniteExpHorn(45.6,1140,25); % call to function FiniteExpHorn
%
% Calculate Ttot = product of segment transfer matrices
%
Ttot=@(k) T1(k)*T2(k)*T3(k);

%
% Enter air density, speed of sound and S1 (throat area) 
% and S99 (mouth area):
%
rho0=0.001205; % g/cm3
c=3.44E4; % cm/sec
S1=5.07;  % 'throat' area = initial segment input area cm2 
S99=1140; % 'mouth area' = final segment output area cm2
%


%
% Piston in infinite baffle impedance choices by ang value
%
    
%
% Enter output impedance of piston in infinite baffle
% Normalized acoustic impedance Zpn of 'piston in infinite baffle'
%
Zpn = @(x) (1-2*besselj(1,x)./x)+1i*2*struveh1approx(x)./x; 
%
% Choose correct output impedance based on value of ang
%
switch ang % check value of ang used to choose which case below
    case 4.0 % case for value of ang = 4.0 
    mult = sqrt(0.5);
    angstr=[num2str(4.0) ' pi']; % develop text string for later plot
% Zp = @(k) (rho0*c/S99)*Zpn(2*k*sqrt(0.5)*sqrt(S99/pi)); % ang=4.0*pi 
    case 2.0 % case for value of ang = 2.0
    mult = 1.0;
    angstr=[num2str(2.0) ' pi']; % develop text string for later plot
% Zp = @(k) (rho0*c/S99)*Zpn(2*k*sqrt(1)*sqrt(S99/pi));   % ang=2.0*pi
    case 1.0 % case for value of ang = 1.0
    mult = sqrt(2);
    angstr=[num2str(1.0) ' pi']; % develop text string for later plot
% Zp = @(k) (rho0*c/S99)*Zpn(2*k*sqrt(2)*sqrt(S99/pi));   % ang=1.0*pi 
    case 0.5 % case for value of ang = 0.5
    mult = sqrt(4);
    angstr=[num2str(0.5) ' pi']; % develop text string for later plot
% Zp = @(k) (rho0*c/S99)*Zpn(2*k*sqrt(4)*sqrt(S99/pi));   % ang=0.5*pi 
    otherwise % default case is ang = 2.0
    disp('Warning: Improper ang value input, ang = 2.0 (default) chosen')
    mult = 1.0;
    angstr=[num2str(2.0) ' pi']; % develop text string for later plot
% Zp = @(k) (rho0*c/S99)*Zpn(2*k*sqrt(1)*sqrt(S99/pi));   % ang=2.0*pi   
end
%
% Calculate output impedance using correct value of mult determined
% above by testing ang value
%
Zp = @(k) (rho0*c/S99)*Zpn(2*k*mult*sqrt(S99/pi));                                                    
%
%
% Calculate the individual Ttot matrix elements
%
Ttot11 = @(k) (Ttot(k)*[1;0]).'*[1;0];
Ttot12 = @(k) (Ttot(k)*[0;1]).'*[1;0];
Ttot21 = @(k) (Ttot(k)*[1;0]).'*[0;1];
Ttot22 = @(k) (Ttot(k)*[0;1]).'*[0;1];
%

%
% Calculate input impedance Za1(k)
%
Za1 = @(k) (Ttot11(k)*Zp(k)+Ttot12(k))/(Ttot21(k)*Zp(k)+Ttot22(k));
%

%
% Develop plot and output function for Za1
%
plotZa1=[];
for f=fLow:delf:fHigh;
plotZa1=cat(1,plotZa1,Za1(2*pi*f/c));
end
%

%
% Develop matrices realZa1 and imagZa1 used to hold Za1 data 
% that is output to Matlab workspace
%
f=fLow:delf:fHigh;
realZa1=[f;(S1/(rho0*c))*real(plotZa1)']';
imagZa1=[f;(S1/(rho0*c))*imag(plotZa1)']';

%
% Now plot real (blue plot) and imaginary (red plot) part
% of acoustic input impedance Za1 on semilog plot
%
figure
semilogx(f,(S1/(rho0*c))*real(plotZa1))
hold on
grid on
semilogx(f,(S1/(rho0*c))*imag(plotZa1),'r')
%
% Add Text to Plot Title to identify ang value
%
%['ang = ' angstr];
%
title({'Normalized Input Acoustic Impedance',['Matlab Calculation: ang = ' angstr]})
legend('Resistive','Reactive','Location','SouthWest')
xlabel('frequency  f')
ylabel('Normalized Input Acoustic Impedance')
% ylim([0 1.2]) % can be used to set vertical scale if so desired
xlim([fLow fHigh])
%

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%    Functions used above %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Horn Segment Functions
%
%%%%%% Cylindrical Segment %%%%%%%
%
function [ Tcyl ] = FiniteCylHorn( S1,L )
% FiniteCylHorn Calculates the Transfer Matrix for a finite cylindrical horn
% This program calculates the acoustical impedance transfer matrix
% Texp of a finite cylindrical horn of input area S1, output
% area S2=S1 and length L.

% Enter air density and speed of sound:
rho0=0.001205;
c=3.44E4;

% Calculate transfer matrix Tcyl
Tcyl= @(k) [cos(k*L)...
    1i*rho0*c*sin(k*L)/S1;...
    1i*S1*sin(k*L)/(rho0*c)...
    cos(k*L)];
end
%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%%%%%% Conical Segment %%%%%%%
%
function [ Tcon ] = FiniteConHorn( S1,S2,L )
% FiniteConHorn Calculates the Transfer Matrix for a finite conical horn
% This program calculates the acoustical impedance transfer matrix
% Texp of a finite conical horn of input area S1, output
% area S2 and length L.

% Enter air density and speed of sound:
rho0=0.001205;
c=3.44E4;

% x1, x2 parameters, etc.
x1 = L/(sqrt(S2/S1)-1);
x2 = sqrt(S2/S1)*L/(sqrt(S2/S1)-1);

% Calculate transfer matrix Tcon
Tcon= @(k) [sqrt(S2/S1)*(cos(k*L) - 1/(k*x2)*sin(k*L))...
    1i.*rho0.*c.*sin(k.*L)./sqrt(S1*S2);...
    1i*sqrt(S1*S2)*((1/(k*x2)-1/(k*x1))*cos(k*L)+(1+1/(k^2*x1*x2))*sin(k*L))/(rho0*c)...
    sqrt(S1/S2)*(cos(k*L) + (1/(k*x1))*sin(k*L))];
end
%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%%%%%% Exponential Segment %%%%%%%
%
function [ Texp ] = FiniteExpHorn( S1,S2,L )
% FiniteExpHorn Calculates the Transfer Matrix for a finite exp horn
% This program calculates the acoustical impedance transfer matrix
% Texp of a finite exponential horn of input area S1, output
% area S2 and length L.

% Enter air density and speed of sound:
rho0=0.001205;
c=3.44E4;

% m parameter, cutoff frequency, etc.
m=log(S2/S1)/L;
fc=m*c/(4*pi); % not used in program, but available for printout
theta= @ (k) asin(0.5*m/k);

% Calculate transfer matrix Texp
Texp= @(k) [sqrt(S2/S1)*(cos(k*L*cos(theta(k))) - tan(theta(k))*sin(k*L*cos(theta(k))))...
    1i*rho0*c*sec(theta(k))*sin(k*L*cos(theta(k)))/sqrt(S1*S2);...
    1i*sqrt(S1*S2)*sec(theta(k))*sin(k*L*cos(theta(k)))/(rho0*c)...
    sqrt(S1/S2)*(cos(k*L*cos(theta(k))) + tan(theta(k))*sin(k*L*cos(theta(k))))];

end
%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Calculate Struve Function needed for piston in baffle impedance
%
function output = struveh1approx(x)
% Struve H1 approximation obtained from paper by Aarts and Janssen (JASA,
% 2003)
%
% Robert J. McGough
% Michigan State University
% 18 Dec 2005

output = 2/pi - besselj(0, x) + (16/pi - 5) * sin(x) ./ x + (12 - 36/pi) .* (1 - cos(x)) ./ x .^2;
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
end


